BrokerXP = LibStub("AceAddon-3.0"):NewAddon("Broker_XP", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Broker_XP", true)

local bxp = BrokerXP
local time = time
local defaults = {
  profile = {
    dataValue = "XPHour",
  }
}

local xptypes = {
          XPHour = L["XP gain rate (session)"],
          XPHourLevel = L["XP gain rate (level)"],
          XP = L["Current XP"],
          XPpcnt = L["Percentage XP"],
          XPtotal = L["XP gained and total for level"],
          LastLevelGainTime = L["Last level gained"],
}

local options = {
    name = L["Broker_XP"],
    handler = bxp,
    type = "group",
  	get = function(info) return bxp.db[info[#info]] end,
  	set = function(info, value) bxp.db[info[#info]] = value end,
    args = {
      dataValue = {
        name = L["Main Value"],
        desc = L["Select the primary value to show"],
        type = "select",
        values = xptypes,
      },
    },
}

function BrokerXP:OnInitialize()
  self.db_root = LibStub("AceDB-3.0"):New("Broker_XPDB",defaults)
  self.db=self.db_root.profile
	local acreg = LibStub("AceConfigRegistry-3.0")
	acreg:RegisterOptionsTable("Broker_XP", options)
	acreg:RegisterOptionsTable("Broker_XP Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db_root))
	local acdia = LibStub("AceConfigDialog-3.0")
	acdia:AddToBlizOptions("Broker_XP", L["Broker_XP"])
	acdia:AddToBlizOptions("Broker_XP Profiles", L["Profiles"], "Broker_XP")
	
	self.levelledXP = 0
	self.sessionXP = 0
	self.playedTimeOffset = time()
	self.lastLevelTimeOffset = time()
	self.sessionStartTime = time()
end

local lastXP, lastMobXP

function BrokerXP:OnEnable()
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("TIME_PLAYED_MSG")

  RequestTimePlayed()

	self:ScheduleRepeatingTimer("UpdateText",1)
	
	if not self.sessionStartXP then
		self.sessionStartXP = UnitXP("player")
	end
	lastXP = self.sessionStartXP
end

function BrokerXP:OnDisable()
	self:UnregisterEvent("PLAYER_XP_UPDATE")
	self:UnregisterEvent("PLAYER_LEVEL_UP")
	self:UnregisterEvent("TIME_PLAYED_MSG")
end

function BrokerXP:PLAYER_XP_UPDATE()
	local newXP = UnitXP("player")
  lastMobXP = newXP - lastXP
	lastXP = newXP
	if lastMobXP < 0 then
		lastMobXP = 0
	end
	self.sessionXP = newXP - self.sessionStartXP + self.levelledXP
end

function BrokerXP:PLAYER_LEVEL_UP()
	self.levelledXP = self.levelledXP + UnitXPMax("player") - self.sessionStartXP
	self.sessionStartXP = 0
	self.lastLevelTimeOffset = time()
end

function BrokerXP:TIME_PLAYED_MSG()
	self.playedTimeOffset = time() - arg1
	self.lastLevelTimeOffset = time() - arg2
end

function BrokerXP:UpdateText()
  local typ=self.db.dataValue
  local fmt,unit,value,value2=self['meth_'..typ](self)
  if unit then
    self.obj.value=value
    self.obj.unit=unit
  else
    self.obj.value=nil
    self.obj.unit=nil
  end
  self.obj.text=format(fmt,value,value2)
end

function BrokerXP:meth_XP()
  return "%d XP", "XP", UnitXP("player")
end

function BrokerXP:meth_XPpcnt()
  return "%.2f %%", "%", (UnitXP("player")/UnitXPMax("player")) *100
end

function BrokerXP:meth_XPtotal()
  local xp,max=UnitXP("player"),UnitXPMax("player")
  return "%d/%d XP",nil,xp,max
end

function BrokerXP:meth_XPHour()
  return "%.1f XP/hour","XP/hour",self.sessionXP / (time() - self.sessionStartTime) * 3600
end

function BrokerXP:meth_XPHourLevel()
  return "%.1f XP/hour","XP/hour",UnitXP("player") / (time() - self.lastLevelTimeOffset) * 3600
end

function BrokerXP:meth_LastLevelGainTime()
  return "%.1f hours ago", "hours ago", (time() - self.lastLevelTimeOffset) / 3600
end

function BrokerXP:Tooltip(tooltip)
  tooltip:AddLine(L["Broker_XP"])
  for k,p in pairs(xptypes) do
    local fmt,_,value,value2=self["meth_"..k](self)
    tooltip:AddLine(format("%s: "..fmt,p,value,value2))
  end
end

local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
if ldb then 
	BrokerXP.obj=ldb:NewDataObject("Broker_XP", {
		type = "data source",
		text = "",
--		icon = "Interface\\Icons\\INV_Elemental_Mote_Nether",
		OnClick = function(self, button)
			InterfaceOptionsFrame_OpenToCategory(L["Broker_XP"])
		end,
		OnTooltipShow = function(tooltip)
		  BrokerXP:Tooltip(tooltip)
		end,
	})
end

